# -*- coding: utf-8 -*-

import netCDF4
import pandas as pd
import numpy as np
import os

from datetime import datetime, timezone
from functions import log, load_qa

from envass import qualityassurance

class secchi(object):
    def __init__(self):
        self.id = ""
        self.folder = ""
        self.data = {}
        self.general_attributes = {
            "institution": "EPFL",
            "source": "Secchi Depth Measurements",
            "references": "LéXPLORE common instruments camille.minaudo@epfl.ch>",
            "history": "See history on Renku",
            "conventions": "CF 1.7",
            "comment": "Secchi depths collected on Lexplore Platform in Lake Geneva",
            "title": "Lexplore Secchi Depth Measurements"
        }
        self.dimensions = {
            'time': {'dim_name': 'time', 'dim_size': None}
        }
        self.variables = {
            'time': {'var_name': 'time', 'dim': ('time',), 'unit': 'seconds since 1970-01-01 00:00:00',
                     'longname': 'time', "quality_check":{"numeric":True,"bounds":[1514764800,datetime.now().timestamp()]}},
            'secchi': {'var_name': 'secchi', 'dim': ('time',), 'unit': 'm', 'longname': 'secchi depth',},
            'lowering': {'var_name': 'lowering', 'dim': ('time',), 'unit': 'm', 'longname': 'lowering depth',},
            'raising': {'var_name': 'raising', 'dim': ('time',), 'unit': 'm', 'longname': 'raising depth',},
        }

    def to_NetCDF(self, folder, prefix):
        log("Writing "+prefix+" data to NetCDF", 3)

        dt = datetime.utcfromtimestamp(np.nanmin(self.data["time"])).strftime('%Y%m%d_%H%M%S')

        if not os.path.exists(folder):
            os.makedirs(folder)
        filename = "_".join([prefix, "SECCHI", dt+".nc"])
        self.outfile = filepath = os.path.join(folder, filename)
        # Check if file already exists
        if os.path.isfile(self.outfile):
            #Open file in 'append' mode
            nc = netCDF4.Dataset(self.outfile,mode='a',format='NETCDF4')
            
            # Check if last timestamp has already been added, if not data will be added
            if self.data["time"][-1] in nc.variables['time'][:]:
                print("Duplicated run, no data added")
                nc.close() 
                return
            
            else:
                # Get index of last saved timestamp
                time_file = np.array(nc.variables['time'][:])
                idx = np.where(self.time==time_file[-1])[0][0] + 1
                
                # Add data to coordinate variable 'time'
                time = nc.variables['time']
                time[:] = np.append(time[:], self.time[idx:])
                
                for key, values in self.variables.items():
                    self.addDataNetCDF(np.array(self.data[key]),idx,nc,**values)
                # Close NetCDF file
                nc.close()
        else: 
            nc = netCDF4.Dataset(filepath, mode='w', format='NETCDF4')

            for key in self.general_attributes:
                setattr(nc, key, self.general_attributes[key])

            for key, values in self.dimensions.items():
                nc.createDimension(values['dim_name'], values['dim_size'])

            for key, values in self.variables.items():
                var = nc.createVariable(values["var_name"], np.float64, values["dim"], fill_value=np.nan)
                var.units = values["unit"]
                var.long_name = values["longname"]
                var[:] = self.data[key]

            # Close NetCDF file
            nc.close()
            log("Successfully wrote NetCDF file", 3)

    def mask_data(self):
        log("Masking flagged data", 3)
        for var in self.variables:
            if "_qual" not in var:
                idx = self.data[var+"_qual"] > 0
                self.data[var][idx] = np.nan

    def quality_flags(self, file_path, simple=True):
        qa = load_qa(file_path)
        var_name = qa.keys()
        for var in var_name:
            if var in qa:
                if simple:
                    qa_arr_simple = qualityassurance(np.array(self.data[var]), np.array(self.data["time"]), **qa[var]["simple"])
                    qa = np.copy(qa_arr_simple)
                else:
                    qa_arr_adv = qualityassurance(np.array(self.data[var]), np.array(self.data["time"]), **qa[var]["advanced"])
                    qa = qa_arr_simple+qa_arr_adv
                self.data[var + "_qual"] = qa

    def read_data(self, folder):
        if os.path.isfile(os.path.join(folder, "Secchi_registration.csv")):
            file = os.path.join(folder, "Secchi_registration.csv")
        else:
            log("Cannot find Secchi file", 3)
            return False
        log("Reading Secchi data from: "+file, 3)
        try:
            df = pd.read_csv(file, sep=";", header=None, encoding = "ISO-8859-1")
            df.columns = ["id","date","hour","lowering","raising","secchi","person"]
            df["time"] = (df["date"] + df["hour"]).apply(lambda x: datetime.timestamp(datetime.strptime(x, '%Y-%m-%d%H:%M').replace(tzinfo=timezone.utc)))

            for variable in self.variables:
                self.data[variable] = np.array(df[variable])
            log("Successfully read data", 3)
        except Exception as e:
            log("Failed to parse data", e)
            return False
        return True